package com.huoran.competition.utils.oss;

import cn.hutool.core.util.IdUtil;
import com.aliyun.oss.ClientException;
import com.aliyun.oss.OSS;
import com.aliyun.oss.OSSClientBuilder;
import com.aliyun.oss.OSSException;
import com.aliyun.oss.model.*;
import com.huoran.common.entity.FilesResult;
import com.huoran.common.utils.ConstantPropertiesUtil;
import org.apache.commons.lang.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.util.List;

public class AliyunOssUtil {

    /**
     * 创建存储空间
     *
     * @param ossClient oss客户端
     */
    public static void createBucket(OSS ossClient) {
        // 判断存储空间是否存在，不存在，则创建
        if (!ossClient.doesBucketExist(ConstantPropertiesUtil.BUCKET_NAME)) {
            // 创建CreateBucketRequest对象,设置仓库名称
            CreateBucketRequest bucketRequest = new CreateBucketRequest(ConstantPropertiesUtil.BUCKET_NAME);
            // 设置存储空间的权限为公共读，默认是私有。
            bucketRequest.setCannedACL(CannedAccessControlList.PublicRead);
            // 此处以设置存储空间的存储类型为标准存储为例。
            // createBucketRequest.setStorageClass(StorageClass.Standard);
            // 默认情况下，数据容灾类型为本地冗余存储，即DataRedundancyType.LRS。如果需要设置数据容灾类型为同城冗余存储，请替换为DataRedundancyType.ZRS。
            // createBucketRequest.setDataRedundancyType(DataRedundancyType.ZRS)
            // 创建存储空间。
            ossClient.createBucket(bucketRequest);

        }
    }

    /**
     * 上传文件
     *
     * @param file 文件
     * @return 返回文件上传结果
     * @throws OSSException    oss异常
     * @throws ClientException 客户端异常
     * @throws IOException     io异常
     */
    public static FilesResult uploadFile(MultipartFile file, String tableName) throws OSSException, ClientException, IOException {

        //获取访问key
        String endPoint = ConstantPropertiesUtil.END_POINT;
        String accessKeyId = ConstantPropertiesUtil.ACCESS_KEY_ID;
        String accessKeySecret = ConstantPropertiesUtil.ACCESS_KEY_SECRET;
        //创建ossClient
        OSS ossClient = new OSSClientBuilder().build(endPoint, accessKeyId, accessKeySecret);

        FilesResult result = new FilesResult();
        // 创建存储空间
        createBucket(ossClient);

        //获取上传文件流
        InputStream inputStream = file.getInputStream();
        //获取当前日期构建日期路径：PythonDataExcel/文件名
//        String currentDate = new DateTime().toString("yyyyMMdd");
        //获取上传的文件名称(此处filename直接取的TableName)
//        String fileName = file.getOriginalFilename();

        String fileName = tableName;
        result.setOriginalFileName(StringUtils.substringBefore(fileName,"."));

        // 获取文件尾缀
        assert fileName != null;
        String ext = fileName.substring(fileName.lastIndexOf("."));
        // 获取文件类型，大小
        String fileType = getFileType(ext);
        long fileSize = file.getSize();

        result.setFileType(fileType);
        result.setFileSize(String.valueOf(fileSize));
        //获取随机值
        String snowflakeId = IdUtil.getSnowflake(6, 4).nextIdStr();
        //文件名：雪花算法.文件扩展名
//        String newName = snowflakeId + ext;
        String sort = ext.substring(ext.lastIndexOf(".") + 1);
        result.setFileType(sort);
        //组合新的文件名
        String fileUrl = "PythonDataExcel/" + fileName;
        result.setOssFileName(fileUrl);
        // 设置上传时存储类型与访问权限
        ObjectMetadata metadata = new ObjectMetadata();
        // 指定该Object被下载时的网页的缓存行为
        metadata.setCacheControl("no-cache");
        // 指定该Object下设置Header
        metadata.setHeader("Pragma", "no-cache");
        // 指定该Object被下载时的内容编码格式
        metadata.setContentEncoding("utf-8");
        // 文件的MIME，定义文件的类型及网页编码，决定浏览器将以什么形式、什么编码读取文件。如果用户没有指定则根据Key或文件名的扩展名生成，
        // 如果没有扩展名则填默认值application/octet-stream
        metadata.setContentType(fileType);
        // 指定该Object被下载时的名称（指示MINME用户代理如何显示附加的文件，打开或下载，及文件名称）
        metadata.setContentDisposition("filename/filesize=" + file.getName() + "/" + fileSize + "Byte.");

        //上传文件 (上传文件流的形式)
        ossClient.putObject(ConstantPropertiesUtil.BUCKET_NAME, fileUrl, inputStream, metadata);
        // 关闭OSSClient。
        ossClient.shutdown();

        // 拼接文件url地址，返回控制层
        String tempEndpoint = ConstantPropertiesUtil.END_POINT;
        String endpoint = tempEndpoint.substring(tempEndpoint.lastIndexOf("/") + 1, tempEndpoint.length());
        String url = "https://" + ConstantPropertiesUtil.BUCKET_NAME + "." + endpoint + "/" + fileUrl;

        result.setFileUrl(url);

        // 关闭OSSClient。
        ossClient.shutdown();

        return result;
    }

    /**
     * 文件下载
     *
     * @param response   响应对象
     * @param ossClient  oss客户端
     * @param objectName 文件名
     */
    public static void downloadFiles(HttpServletResponse response, OSS ossClient, String objectName) throws IOException {

        // ossObject包含文件所在的存储空间名称、文件名称、文件元信息以及一个输入流。
        OSSObject ossObject = ossClient.getObject(ConstantPropertiesUtil.BUCKET_NAME, objectName);
        InputStream input = null;
        OutputStream outputStream = null;
        try {
            // 获取文件名
            String fileName = objectName.substring(objectName.lastIndexOf("/") + 1);
            //获取输入流
            input = ossObject.getObjectContent();
            // 获取OutputStream输出流
            outputStream = response.getOutputStream();
            byte[] buffer = new byte[input.available()];

            response.setHeader("content-Type", "application/octet-stream");
            response.setHeader("Content-disposition", "attachment;filename=" + URLEncoder.encode(fileName, "UTF-8"));
            response.flushBuffer();

            for (int length = 0; (length = input.read(buffer)) > 0; ) {
                outputStream.write(buffer, 0, length);
            }

        } catch (IOException e) {
            e.printStackTrace();
            throw e;
        } finally {
            if (null != outputStream) {
                try {
                    outputStream.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            if (null != input) {
                try {
                    input.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            // 关闭OSSClient。
            ossClient.shutdown();
        }

    }

    /**
     * *** 谨慎操作 ***
     * 删除存储空间
     *
     * @param bucketName 存储空间名称
     */
    public static void deleteBucket(String bucketName) {
        String endPoint = ConstantPropertiesUtil.END_POINT;
        String accessKeyId = ConstantPropertiesUtil.ACCESS_KEY_ID;
        String accessKeySecret = ConstantPropertiesUtil.ACCESS_KEY_SECRET;
        // 创建OSSClient实例。
        OSS ossClient = new OSSClientBuilder().build(endPoint, accessKeyId, accessKeySecret);
        // 删除存储空间。
        ossClient.deleteBucket(bucketName);
        // 关闭OSSClient。
        ossClient.shutdown();
    }

    /**
     * 创建文件夹
     *
     * @param ossClient  oss连接
     * @param bucketName 存储空间
     * @param folder     文件夹名，如 "qj_nanjing/"
     * @return 返回创建的文件夹名
     */
    public static String createFolder(OSS ossClient, String bucketName, String folder) {
        // 判断文件夹是否存在，不存在则创建
        if (!ossClient.doesObjectExist(bucketName, folder)) {
            // 创建文件夹
            ossClient.putObject(bucketName, folder, new ByteArrayInputStream(new byte[0]));
            // 得到文件夹名
            OSSObject object = ossClient.getObject(bucketName, folder);
            return object.getKey();
        }
        return folder;
    }

    /**
     * 批量删除文件。key等同于ObjectName，表示删除OSS文件时需要指定包含文件后缀在内的完整路径，例如abc/efg/123.jpg。
     *
     * @param keys 文件完整路径
     */
    public static List<String> deleteFiles(List<String> keys) {
        String endPoint = ConstantPropertiesUtil.END_POINT;
        String accessKeyId = ConstantPropertiesUtil.ACCESS_KEY_ID;
        String accessKeySecret = ConstantPropertiesUtil.ACCESS_KEY_SECRET;
        String bucketName = ConstantPropertiesUtil.BUCKET_NAME;
        // 创建OSSClient实例。
        OSS ossClient = new OSSClientBuilder().build(endPoint, accessKeyId, accessKeySecret);

        DeleteObjectsResult deleteObjectsResult = ossClient.deleteObjects(new DeleteObjectsRequest(bucketName).withKeys(keys));
        List<String> deletedObjects = deleteObjectsResult.getDeletedObjects();
        // 关闭OSSClient。
        ossClient.shutdown();
        //返回删除成功文件列表
        return deletedObjects;
    }

    /**
     * 获取文件类型
     *
     * @param fileExtension 文件后缀
     * @return 返回类型
     */
    public static String getFileType(String fileExtension) {
        if (".bmp".equalsIgnoreCase(fileExtension)) {
            return "image/bmp";
        }
        if (".gif".equalsIgnoreCase(fileExtension)) {
            return "image/gif";
        }
        if (".jpeg".equalsIgnoreCase(fileExtension) || ".jpg".equalsIgnoreCase(fileExtension) || ".png".equalsIgnoreCase(fileExtension)) {
            return "image/jpg";
        }
        if (".html".equalsIgnoreCase(fileExtension)) {
            return "text/html";
        }
        if (".txt".equalsIgnoreCase(fileExtension)) {
            return "text/plain";
        }
        if (".vsd".equalsIgnoreCase(fileExtension)) {
            return "application/vnd.visio";
        }
        if (".ppt".equalsIgnoreCase(fileExtension) || ".pptx".equalsIgnoreCase(fileExtension)) {
            return "application/vnd.ms-powerpoint";
        }
        if (".doc".equalsIgnoreCase(fileExtension) || ".docx".equalsIgnoreCase(fileExtension)) {
            return "application/msword";
        }
        if (".xml".equalsIgnoreCase(fileExtension)) {
            return "text/xml";
        }
        if (".mp4".equalsIgnoreCase(fileExtension) || ".avi".equalsIgnoreCase(fileExtension)) {
            return "video/mp4";
        }
        if (".mpg".equalsIgnoreCase(fileExtension) || ".mpeg".equalsIgnoreCase(fileExtension)) {
            return "video/mpeg";
        }
        if ("pdf".equalsIgnoreCase(fileExtension)) {
            return "application/pdf";
        }
        return "application/octet-stream";
    }

}
