package com.huoran.nakadai.utils;

import cn.hutool.core.date.BetweenFormater;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.io.StreamProgress;
import cn.hutool.http.HttpUtil;
import com.huoran.common.exception.CustomException;
import com.huoran.common.exception.ExceptionEnum;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.mail.javamail.JavaMailSender;
import org.springframework.mail.javamail.MimeMessageHelper;
import org.springframework.stereotype.Component;

import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;
import java.io.File;
import java.util.List;
import java.util.UUID;

/**
 * @Author chen
 * @DATE 2022/6/10 14:23
 * @Version 1.0
 * @Description 邮件发送工具
 */
@Component
public class EmailUtil {

    @Value("${spring.mail.from}") // 从application.yml配置文件中获取
    private String from; // 发送发邮箱地址

    @Value("${email.url}") // 从application.yml配置文件中获取
    private String url; // 发送附件地址

    @Autowired
    private JavaMailSender mailSender;

    /**
     * 发送纯文本邮件信息
     *
     * @param to      接收方
     * @param subject 邮件主题
     * @param content 邮件内容（发送内容）
     */
    public void sendMessage(String to, String subject, String content) {

        MimeMessage mimeMessage = mailSender.createMimeMessage();
        // 创建一个邮件对象
        // SimpleMailMessage msg = new SimpleMailMessage();
        try {
            MimeMessageHelper msg = new MimeMessageHelper(mimeMessage, true);
            msg.setFrom(new InternetAddress(from, "或然科技", "UTF-8")); // 设置发送发
            msg.setTo(to); // 设置接收方
            msg.setSubject(subject); // 设置邮件主题
            msg.setText(content); // 设置邮件内容
            // 发送邮件
            mailSender.send(mimeMessage);
        } catch (Exception e) {
            e.printStackTrace();
            throw new CustomException(ExceptionEnum.ENTER_CORRECT_EMAIL);
        }
    }

    /**
     * 发送带附件的邮件信息
     *
     * @param to      接收方
     * @param subject 邮件主题
     * @param content 邮件内容（发送内容）
//     * @param files   文件数组 // 可发送多个附件
     */
    public void sendMessageCarryFiles(String to, String subject, String content, List<String> urls,List<String> fileName) {
        MimeMessage mimeMessage = mailSender.createMimeMessage();
        try {
            MimeMessageHelper helper = new MimeMessageHelper(mimeMessage, true);
            helper.setFrom(new InternetAddress(from, "或然科技", "UTF-8")); // 设置发送方
            helper.setTo(to); // 设置接收方
            helper.setSubject(subject); // 设置邮件主题
            helper.addTo(to);
            if (fileName.size()<2){
                helper.setText(fileName.get(0)+"已发送至邮箱附件，请注意查收！"); // 设置邮件内容
            }else {
                helper.setText("多个文件已发送至邮箱附件，请注意查收！"); // 设置邮件内容
            }

            if (urls != null && urls.size() > 0) { // 下载文件并发送（多个）
                for (int i = 0; i < urls.size(); i++) {
                    //下载文件到本地
                    downLoadFromUrl(urls.get(i),fileName.get(i));
                    //创建新文件
                    System.out.println(url + fileName.get(i));
                    File file = new File(url + fileName.get(i));
                    helper.addAttachment(file.getName(), file); // 单个附件
                }
            }
            // 发送邮件
            mailSender.send(mimeMessage);
        } catch (Exception e) {
            e.printStackTrace();
            throw new CustomException(ExceptionEnum.ENTER_CORRECT_EMAIL);
        }
    }


    /**
     * @description 从网络URL中下载文件
     * @date 15:33 2021/11/2
     * @params [fileUrl, diskPath]
     */
    public void downLoadFromUrl(String fileUrl, String diskPath) {
        String path;
        if (fileUrl != null) {
            //文件后缀
//            String fileName = fileUrl.substring(fileUrl.lastIndexOf("."));
            try {
                String uuidName = UUID.randomUUID().toString();
                File file = new File(url);
                if (!file.exists()) {
                    //创建文件夹
                    boolean mkdir = file.mkdir();
                    if (!mkdir) {
                        throw new RuntimeException("创建文件夹失败，路径为：" + url);
                    }
                }
                path = url + diskPath;
                String begin = DateUtil.now();
                DateTime beginTime = DateUtil.parse(begin);
                //带进度显示的文件下载
                HttpUtil.downloadFile(fileUrl, FileUtil.file(path), new StreamProgress() {
                    @Override
                    public void start() {
                        System.out.println("开始下载，时间为：" + begin);
                    }

                    @Override
                    public void progress(long progressSize) {
                        System.out.println("已下载：{}"+FileUtil.readableFileSize(progressSize));
                    }

                    @Override
                    public void finish() {
                        String end = DateUtil.now();
                        DateTime endTime = DateUtil.parse(end);
                        long between = DateUtil.between(beginTime, endTime, DateUnit.MS);
                        System.out.println("下载完成，用时：" + DateUtil.formatBetween(between, BetweenFormater.Level.SECOND));
                    }
                });
            } catch (Exception e) {
                System.out.println("下载异常，异常信息为：" + e.getMessage());
            }
        }
    }

    /**
     * 发送带附件的邮件信息
     *
     * @param to      接收方
     * @param subject 邮件主题
     * @param fileUrl 文件链接
     *                // * @param file 单个文件
     */
    public void sendMessageCarryFile(String to, String subject, String fileUrl, String fileName) {

        MimeMessage mimeMessage = mailSender.createMimeMessage();
        try {
            //下载文件到本地
            downLoadFromUrl(fileUrl,fileName);
            //创建新文件
            File file = new File(url + fileName);
            MimeMessageHelper helper = new MimeMessageHelper(mimeMessage, true);
            helper.setFrom(new InternetAddress(from, "或然科技", "UTF-8")); // 设置发送方
            helper.setTo(to); // 设置接收方
            helper.setSubject(subject); // 设置邮件主题
            helper.setText(fileName+"已发送至邮箱附件，请注意查收！"); // 设置邮件内容
            helper.addAttachment(file.getName(), file); // 单个附件
            // 发送邮件
            mailSender.send(mimeMessage);
        } catch (Exception e) {
            e.printStackTrace();
            throw new CustomException(ExceptionEnum.ENTER_CORRECT_EMAIL);
        }
    }

    public String getFrom() {
        return from;
    }

    public void setFrom(String from) {
        this.from = from;
    }
}

