package com.huoran.users.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huoran.api.UserClient;
import com.huoran.common.constant.DelConstant;
import com.huoran.common.response.R;
import com.huoran.common.utils.TokenUtils;
import com.huoran.users.entity.*;
import com.huoran.users.entity.vo.DataPermissionVo;
import com.huoran.users.service.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 角色表 前端控制器
 * </p>
 *
 * @author chen
 * @since 2021-09-29
 */
@Api(tags = "角色管理")
@RestController
@RequestMapping("/users/role")
public class RoleController {

    @Autowired
    private RoleService roleService;

    @Autowired
    private PermissionService permissionService;

    @Autowired
    private RolePermissionService rolePermissionService;

    @Autowired
    private IHrUserAccountService userAccountService;

    @Autowired
    private UserRoleService userRoleService;

    @Autowired
    private UserClient userClient;

    @Autowired
    DataPermissionService dataPermissionService;

    @ApiOperation(value = "角色分页列表查询")
    @GetMapping("/list")
    public R index(
            @ApiParam(name = "page", value = "当前页码", required = true)
            @RequestParam Integer page,
            @ApiParam(name = "size", value = "每页记录数", required = true)
            @RequestParam Integer size,
            @ApiParam(name = "platformId", value = "平台id", required = true)
            @RequestParam Integer platformId,
            @ApiParam(name = "name", value = "角色名称", required = false)
            @RequestParam(required = false) String name,
            @ApiParam(name = "port", value = "端口（pc端为0、小程序端为1）", required = false)
            @RequestParam(required = false) String port, HttpServletRequest request) {
        String accountId = TokenUtils.getIdByJwtToken(request);
        HrUserAccount userAccount = userAccountService.getById(Integer.valueOf(accountId));

        Page<Role> pageParam = new Page<>(page, size);
        QueryWrapper<Role> wrapper = new QueryWrapper<>();
        if (!"".equals(name)) {
            wrapper.like("role_name", name);
        }
        if (!"".equals(port)) {
            wrapper.eq("port", port);
        }
        wrapper.eq("school_id", userAccount.getSchoolId());
        wrapper.eq("platform_id", platformId);
        IPage<Role> rolePage = roleService.page(pageParam, wrapper);
        return R.ok().put("rolePage", rolePage);
    }

    @ApiOperation(value = "获取角色详情")
    @GetMapping("/obtainDetails")
    public R obtainDetails(
            @ApiParam(name = "id", value = "角色id", required = true)
            @RequestParam String id) {
        Role role = roleService.getById(id);
        QueryWrapper<RolePermission> wrapper = new QueryWrapper<>();
        wrapper.eq("role_id", id).select("permission_id");
        List<RolePermission> list = rolePermissionService.list(wrapper);
        List<Integer> permissionList = new ArrayList<>();
        list.forEach(rolePermission -> {
            permissionList.add(rolePermission.getPermissionId());
        });
        List<DataPermissionVo> permissionVoList = dataPermissionService.selectDataPermission(id);
        return R.ok().put("role", role).put("permissionList", permissionList).put("permissionVoList",permissionVoList);
    }

    @ApiOperation(value = "新增或更新角色")
    @PostMapping("/saveOrUpdate")
    public R save(
            @ApiParam(name = "role", value = "角色信息", required = true)
            @Valid @RequestBody Role role, HttpServletRequest request) {
        //编辑判重
        if (role.getId() != null) {
            //名称判重
            boolean isRepeat = roleService.repeat(role);
            if (isRepeat) {
                return R.error("角色已存在");
            }
            //新增判重
        } else {
            String accountId = TokenUtils.getIdByJwtToken(request);
            QueryWrapper<HrUserAccount> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("is_del", DelConstant.NOT_DEL);
            queryWrapper.eq("id", accountId);
            Integer schoolId = userAccountService.getOne(queryWrapper).getSchoolId();
            role.setSchoolId(schoolId);
            QueryWrapper<Role> roleWrapper = new QueryWrapper<>();
            roleWrapper.eq("role_name", role.getRoleName());
            roleWrapper.eq("school_id", role.getSchoolId());
            roleWrapper.eq("platform_id", role.getPlatformId());
            int count = roleService.count(roleWrapper);
            if (count > 0) {
                return R.error("角色已存在");
            }
        }
        roleService.saveOrUpdate(role);
        //分配权限
        permissionService.saveRolePermissionRelationShip(role.getId(), role.getPermissionId());
//        目前中台分配数据权限
        if (role.getPlatformId()==3){
            //分配数据权限
            permissionService.assigningDataPermissions(role.getId(),role.getDataPermissionList());
        }
        return R.ok().put("roleId", role.getId());
    }

    @ApiOperation(value = "批量删除角色")
    @PostMapping("/batchRemove")
    public R batchRemove(
            @ApiParam(name = "roleIds", value = "多个角色id", required = true)
            @RequestParam("roleIds") List<Integer> roleIds) {
        if (roleIds != null && !roleIds.isEmpty()) {
            boolean remove = roleService.batchRemove(roleIds);
            return remove ? R.ok() : R.error();
        } else {
            return R.error();
        }
    }

    //远程调用
    @ApiOperation(value = "删除某用户下的某个角色")
    @PostMapping("/delRoleByAccountId")
    public R delRoleByAccountId(
            @ApiParam(name = "roleId", value = "角色id", required = true) @RequestParam("roleId") Integer roleId,
            @ApiParam(name = "accountId", value = "账号id", required = true) @RequestParam("accountId") Integer accountId) {
        UpdateWrapper<UserRole> updateWrapper = new UpdateWrapper<>();
        updateWrapper.set("is_del", DelConstant.IS_DEL)
                .eq("role_id", roleId)
                .eq("account_id", accountId);
        boolean ret = userRoleService.update(new UserRole(), updateWrapper);
        return ret ? R.ok() : R.error();
    }

    /**
     * 为客户添加超管角色等相关操作
     *
     * @param schoolId  学校id
     * @param accountId 账号id
     */
    @PostMapping("/addCustomerAdmin")
    public boolean addCustomerAdmin(
            @RequestParam("schoolId") Integer schoolId, @RequestParam("accountId") String accountId) {
        return roleService.addCustomerAdmin(schoolId, accountId);
    }

    @ApiOperation(value = "判断该角色是否存在(远程调用，无需调用)")
    @PostMapping("/checkRoleIsExist")
    public Role checkRoleIsExist(@ApiParam(name = "roleName", value = "角色名称", required = true) String roleName,
                                 @ApiParam(name = "schoolId", value = "学校id", required = true) Integer schoolId,
                                 @ApiParam(name = "platformId", value = "平台id", required = true) Integer platformId) {
        return roleService.getRoleInfo(schoolId, roleName,platformId);
    }

    @ApiOperation(value = "根据账号id查询用户的角色(远程调用)")
    @PostMapping("/checkIsSuperTube")
    public boolean checkIsSuperTube(@ApiParam(name = "accountId", value = "账号id", required = true) @RequestParam("accountId") String accountId) {
        return userRoleService.checkIsSuperTube(accountId);
    }

    @ApiOperation(value = "根据账号id查询用户某平台下的全部角色(远程调用)")
    @PostMapping("/getUserAllRole")
    public String getUserAllRole(@ApiParam(name = "accountId", value = "账号id", required = true) @RequestParam("accountId") String accountId,
                                 @ApiParam(name = "platformId", value = "平台id(1.职站 2.数据平台 3.中台)", required = true) @RequestParam("platformId") Integer platformId) {
        return userRoleService.getUserAllRole(Integer.valueOf(accountId), platformId);
    }



    @ApiOperation(value = "根据学校id查询该校下的超管、管理员的账号id(远程调用)")
    @PostMapping("/getAccountIdsBySchoolId")
    public List<Integer> getAccountIdsBySchoolId(@ApiParam(name = "schoolId", value = "学校id", required = true) @RequestParam("schoolId") Integer schoolId,
                                                 @ApiParam(name = "platformId", value = "平台id(1.职站 2.数据平台 3.中台)", required = true) @RequestParam("platformId") Integer platformId) {
        return userRoleService.getAccountIdsBySchoolId(schoolId, platformId);
    }


    @ApiOperation(value = "根据查询用户某平台下的全部角色")
    @PostMapping("/getUserAllRoleByToken")
    public String getUserAllRoleByToken(@ApiParam(name = "platformId", value = "平台id(1.职站 2.数据平台 3.中台)", required = true) @RequestParam("platformId") Integer platformId, @RequestHeader String token) {
        //token查询学校id
        Integer schoolId = userClient.getSchoolIdByAccountId(TokenUtils.getIdByJwtToken(token));

        String accountId = TokenUtils.getIdByJwtToken(token);
        return userRoleService.getUserAllRole(Integer.valueOf(accountId), platformId);
    }

    @ApiOperation(value = "客户重新分配权限")
    @PostMapping("/reassignmentPermissions")
    public R reassignmentPermissions() {
        return roleService.reassignmentPermissions();
    }
}

