package com.huoran.users.utils;

import com.huoran.common.exception.CustomException;
import com.huoran.common.exception.ExceptionEnum;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.mail.javamail.JavaMailSender;
import org.springframework.mail.javamail.MimeMessageHelper;
import org.springframework.stereotype.Component;

import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;
import java.io.File;

/**
 * @Author chen
 * @DATE 2022/6/10 14:23
 * @Version 1.0
 * @Description 邮件发送工具
 */
@Component
public class EmailUtil {

    @Value("${spring.mail.from}") // 从application.yml配置文件中获取
    private String from; // 发送发邮箱地址

    @Value("${email.url}") // 从application.yml配置文件中获取
    private String url; // 发送附件地址

    @Autowired
    private JavaMailSender mailSender;

    /**
     * 发送纯文本邮件信息
     *
     * @param to      接收方
     * @param subject 邮件主题
     * @param content 邮件内容（发送内容）
     */
    public void sendMessage(String to, String subject, String content) {

        MimeMessage mimeMessage = mailSender.createMimeMessage();
        // 创建一个邮件对象
        // SimpleMailMessage msg = new SimpleMailMessage();
        try {
            MimeMessageHelper msg = new MimeMessageHelper(mimeMessage, true);
            msg.setFrom(new InternetAddress(from, "或然科技", "UTF-8")); // 设置发送发
            msg.setTo(to); // 设置接收方
            msg.setSubject(subject); // 设置邮件主题
            msg.setText(content); // 设置邮件内容
            // 发送邮件
            mailSender.send(mimeMessage);
        } catch (Exception e) {
            e.printStackTrace();
            throw new CustomException(ExceptionEnum.ENTER_CORRECT_EMAIL);
        }
    }

    /**
     * 发送带附件的邮件信息
     *
     * @param to      接收方
     * @param subject 邮件主题
     * @param content 邮件内容（发送内容）
     * @param files   文件数组 // 可发送多个附件
     */
    public void sendMessageCarryFiles(String to, String subject, String content, File[] files) {
        MimeMessage mimeMessage = mailSender.createMimeMessage();
        try {
            MimeMessageHelper helper = new MimeMessageHelper(mimeMessage, true);
            helper.setFrom(from); // 设置发送发
            helper.setTo(to); // 设置接收方
            helper.setSubject(subject); // 设置邮件主题
            helper.setText(content); // 设置邮件内容
            if (files != null && files.length > 0) { // 添加附件（多个）
                for (File file : files) {
                    helper.addAttachment(file.getName(), file);
                }
            }
            // 发送邮件
            mailSender.send(mimeMessage);
        } catch (Exception e) {
            e.printStackTrace();
            throw new CustomException(ExceptionEnum.ENTER_CORRECT_EMAIL);
        }
    }

    /**
     * 发送带附件的邮件信息
     *
     * @param to      接收方
     * @param subject 邮件主题
     * @param content 邮件内容（发送内容）
     *                // * @param file 单个文件
     */
    public void sendMessageCarryFile(String to, String subject, String content, String fileName) {
        MimeMessage mimeMessage = mailSender.createMimeMessage();
        try {
            File file = new File(url + fileName);
            MimeMessageHelper helper = new MimeMessageHelper(mimeMessage, true);
            helper.setFrom(new InternetAddress(from, "或然科技", "UTF-8")); // 设置发送方
            helper.setTo(to); // 设置接收方
            helper.setSubject(subject); // 设置邮件主题
            helper.setText(content); // 设置邮件内容
            helper.addAttachment(file.getName(), file); // 单个附件
            // 发送邮件
            mailSender.send(mimeMessage);
        } catch (Exception e) {
            e.printStackTrace();
            throw new CustomException(ExceptionEnum.ENTER_CORRECT_EMAIL);
        }
    }

    public String getFrom() {
        return from;
    }

    public void setFrom(String from) {
        this.from = from;
    }
}

