package com.huoran.websocket.server;

import com.alibaba.fastjson.JSON;
import com.huoran.websocket.constant.WebsocketConst;
import com.huoran.websocket.model.dto.WebsocketMsgDTO;
import com.huoran.websocket.model.entity.OnlineUser;
import com.huoran.websocket.model.entity.RedisOnlineUser;
import com.huoran.websocket.model.vo.SendMsgVO;
import com.huoran.websocket.service.WebsocketService;
import com.huoran.websocket.topic.WebsocketMsgPublisher;
import com.huoran.websocket.util.RedisUtil;
import com.huoran.websocket.util.WebsocketSpringContextUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.websocket.*;
import javax.websocket.server.PathParam;
import javax.websocket.server.ServerEndpoint;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * @author wangsong
 * Websocket 监听类(连接，断点，消息发送等)
 * <p>
 * /websocket/{userId}/{username}   =  /websocket/用户Id/用户名 来连接websocket，该参数会带到每一个监听方法中
 * 同一个账号重复登录, 会被挤下线
 * </P>
 * @ServerEndpoint: socket链接地址
 */
@ServerEndpoint("/websocket/{userId}/{username}")
@Slf4j
@Component
public class WebsocketServer {


    /**
     * 所有用户信息(session + userId + username + createTime  --> 以用户的id为key, 通过用户key来获取用户session进行消息发送)
     */
    private final static Map<String, List<OnlineUser>> CLIENTS = new ConcurrentHashMap<>();

    /**
     * 获取在线用户列表
     *
     * @return {@link Map}<{@link String}, {@link List}<{@link OnlineUser}>>
     */
    public static Map<String, List<OnlineUser>> getClients() {
        return CLIENTS;
    }

    /**
     * 监听连接（有用户连接，立马到来执行这个方法），session 发生变化
     *
     * @param userId   用户id
     * @param username 用户名
     * @param session  当前用户会话
     * @return void
     * @date 2020/6/30 0030 9:28
     */
    @OnOpen
    public void onOpen(@PathParam("userId") String userId, @PathParam("username") String username, Session session) {
        WebsocketService websocketService = WebsocketSpringContextUtil.getBean(WebsocketService.class);
        Integer onlineCount = websocketService.getOnlineCount();
        // 保存到当前服务器
        onlineLocal(userId, username, session);
        onlineRedis(userId, username, session, onlineCount);
        // 给自己发条消息,告诉自己连接成功
        try {
            SendMsgVO sendMsgVO = new SendMsgVO();
            sendMsgVO.setFrom(userId);
            sendMsgVO.setUsername(username);
            sendMsgVO.setTo(userId);
            sendMsgVO.setContent("欢迎您: " + username);
            sendMsgVO.setExtras(null);
            sendMsgVO.setOnlineNum(websocketService.getOnlineCount());
            session.getBasicRemote().sendText(JSON.toJSONString(sendMsgVO));
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }


    /**
     * 监听断开连接（有用户退出，会立马到来执行这个方法）
     *
     * @param userId   用户id
     * @param username 用户名
     * @param session  当前用户会话
     */
    @OnClose
    public void onClose(@PathParam("userId") String userId, @PathParam("username") String username, Session session) {
        // 移除本地在线状态
        offLineLocal(userId, session);
        // 移除 redis 在线状态
        offLineRedis(userId, session);
        // 关闭 (避免没有正常关闭主动在关闭一次)
        try {
            session.close();
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        WebsocketService websocketService = WebsocketSpringContextUtil.getBean(WebsocketService.class);
        log.info(username + ":已离线！ 当前在线人数" + websocketService.getOnlineCount());
    }


    /**
     * 异常停止
     *
     * @param userId   用户id
     * @param username 用户名
     * @param session  当前用户会话
     * @param e        异常信息
     */
    @OnError
    public void onError(@PathParam("userId") String userId, @PathParam("username") String username, Session session, Throwable e) {
        log.info("服务端发生了错误" + e);
        // 关闭 (避免没有正常关闭主动在关闭一次)
        try {
            session.close();
        } catch (IOException e1) {
            throw new RuntimeException(e1);
        }
    }


    /**
     * 监听消息发送（收到客户端的消息立即执行）
     *
     * @param userId   用户id
     * @param username 用户名
     * @param message  传递的消息内容, json数据( to=接收人用户Id  (目标ID,逗号分隔) || content=内容  || content=消息类型)
     * @param session  当前用户会话
     *                 <p>
     *                 // 前端发送内容格式
     *                 ....
     *                 // 拼接参数
     *                 let message = {"content": "测试发送消息", "to": "ALL"}
     *                 ....
     */
    @OnMessage
    public void onMessage(@PathParam("userId") String userId, @PathParam("username") String username, String message, Session session) {
        WebsocketMsgPublisher websocketMsgPublisher = WebsocketSpringContextUtil.getBean(WebsocketMsgPublisher.class);
        WebsocketService websocketService = WebsocketSpringContextUtil.getBean(WebsocketService.class);
        // 请求参数（接收人+发送内容）
        try {
            System.out.println(message);
            WebsocketMsgDTO sendMsgDTO = JSON.parseObject(message, WebsocketMsgDTO.class);
            // 接收到消息 发送
            SendMsgVO sendMsgVO = new SendMsgVO();
            sendMsgVO.setFrom(userId);
            sendMsgVO.setUsername(username);
            sendMsgVO.setTo("all");
            sendMsgVO.setContent(sendMsgDTO.getContent());
            sendMsgVO.setExtras(null);
            sendMsgVO.setOnlineNum(websocketService.getOnlineCount());
            websocketMsgPublisher.sendMsg(sendMsgVO);
        } catch (Exception e) {
            // 给发送人 推送发送错误信息
            log.error("发送的消息格式错误");
            // 发送消息
            String content = "发送的消息格式错误";
            SendMsgVO sendMsgVO = new SendMsgVO();
            sendMsgVO.setFrom(userId);
            sendMsgVO.setUsername(username);
            sendMsgVO.setTo(userId);
            sendMsgVO.setContent(content);
            sendMsgVO.setExtras(null);
            sendMsgVO.setOnlineNum(websocketService.getOnlineCount());
            websocketMsgPublisher.sendMsg(sendMsgVO);
        }
    }


    /**
     * 上线 保存在线状态到本地
     *
     * @param userId 用户id
     * @param session 会话
     */
    private void onlineLocal(String userId, String username, Session session) {
        if (CLIENTS.containsKey(userId)) {
            // 保存已存在用户的 session 会话
            CLIENTS.get(userId).add(new OnlineUser(userId, username, session));
        } else {
            // 保存用户 session 会话
            ArrayList<OnlineUser> onlineUsers = new ArrayList<>();
            onlineUsers.add(new OnlineUser(userId, username, session));
            CLIENTS.put(userId, onlineUsers);
        }
    }

    /**
     * 上线 保存在线状态 Redis
     *
     * @param userId 用户id
     * @param session 会话
     */
    private void onlineRedis(String userId, String username, Session session, Integer onlineCount) {
        RedisUtil redisUtil = WebsocketSpringContextUtil.getBean(RedisUtil.class);
        // 保存到 redis
        boolean isUser = redisUtil.hHasKey(WebsocketConst.WEBSOCKET_CACHE_KEY, userId);
        if (isUser) {
            // 已存在, 多地登录追加重复登录用户信息到 redis 中
            List<RedisOnlineUser> redisOnlineUsers = (List<RedisOnlineUser>) redisUtil.hGet(WebsocketConst.WEBSOCKET_CACHE_KEY, userId);
            redisOnlineUsers.add(new RedisOnlineUser(userId, username, session.getId()));
            redisUtil.hPut(WebsocketConst.WEBSOCKET_CACHE_KEY, userId, redisOnlineUsers);
            log.info("重复登录 ！sessionId：{} userId：{} -> {} userName：{} 当前在线人数:{}", session.getId(), redisOnlineUsers.size(), userId, username, onlineCount);
        } else {
            // 保存用户到 redis 中
            ArrayList<RedisOnlineUser> redisOnlineUsers = new ArrayList<>();
            redisOnlineUsers.add(new RedisOnlineUser(userId, username, session.getId()));
            redisUtil.hPut(WebsocketConst.WEBSOCKET_CACHE_KEY, userId, redisOnlineUsers);
            log.info("有新连接加入！sessionId：{} userId：{} userName：{} 当前在线人数:{}", session.getId(), userId, username, onlineCount + 1);
        }
    }


    /**
     * websocket 下线处理
     * session 连接状态
     */
    private void offLineLocal(String userId, Session session) {
        // 剔除下线用户 (当前服务中剔除)
        List<OnlineUser> onlineUsers = CLIENTS.get(userId);
        // 删除当前 session 连接
        for (OnlineUser onlineUser : onlineUsers) {
            if (onlineUser.getSession() == session) {
                onlineUsers.remove(onlineUser);
                break;
            }
        }
        // 如果同一用户已不存在任意连接
        if (onlineUsers.size() == 0) {
            CLIENTS.remove(userId);
        } else {
            CLIENTS.put(userId, onlineUsers);
        }
    }

    /**
     * 从 redis 中移除用户在线状态
     *
     * @param userId 用户id
     * @param session 会话
     */
    private void offLineRedis(String userId, Session session) {
        RedisUtil redisUtil = WebsocketSpringContextUtil.getBean(RedisUtil.class);
        List<RedisOnlineUser> redisOnlineUsers = (List<RedisOnlineUser>) redisUtil.hGet(WebsocketConst.WEBSOCKET_CACHE_KEY, userId);
        for (RedisOnlineUser redisOnlineUser : redisOnlineUsers) {
            if (redisOnlineUser.getSessionId().equals(session.getId())) {
                redisOnlineUsers.remove(redisOnlineUser);
                break;
            }
        }
        if (redisOnlineUsers.size() == 0) {
            redisUtil.hDelete(WebsocketConst.WEBSOCKET_CACHE_KEY, userId);
        } else {
            redisUtil.hPut(WebsocketConst.WEBSOCKET_CACHE_KEY, userId, redisOnlineUsers);
        }
    }


    /**
     * 组装 消息通知需要的内容格式
     *
     * @param title  通知标题
     * @param message 通知消息
     */
    private String assembleNoticeMsg(String title, String message) {
        Map<String, String> moticeMsgData = new HashMap<>(2);
        Map<String, String> moticeContent = new HashMap<>(4);
        moticeContent.put("title", title);
        moticeContent.put("message", message);
        moticeMsgData.put("content", JSON.toJSONString(moticeContent));
        return JSON.toJSONString(moticeMsgData);
    }
}